/*
 * linux/fs/cbm/ioctl.c
 *
 * Written 1995  Dan Fandrich <dan@fch.wimsey.bc.ca>
 *
 * Modified from ext2 code,
 * Copyright (C) 1993, 1994  Remy Card (card@masi.ibp.fr)
 *                           Laboratoire MASI - Institut Blaise Pascal
 *                           Universite Pierre et Marie Curie (Paris VI)
 */

#include <asm/segment.h>

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/cbm_fs.h>
#include <linux/cbm_fs_i.h>
#include <linux/cbm_fs_sb.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/mm.h> /* TEST */

#define PRINTK(a) /*printk a*/		/* debugging */

int cbm_ioctl (struct inode * inode, struct file * filp, unsigned int cmd,
		unsigned long arg)
{
	int err;
	unsigned long file_type;

	PRINTK( ("cbm_ioctl: cmd = 0x%x, arg = %lu\n", cmd, arg));

	switch (cmd) {

	/* Get CBM file type byte which includes closed & locked attributes */
	case CBM_IOC_GETFTYPE:
		if ((err = verify_area (VERIFY_WRITE, (long *) arg, sizeof(long))))
			return err;
		put_fs_long ((unsigned long) CBM_I(inode)->i_type, (long *) arg);
		return 0;

	/* Set CBM file type byte */
	case CBM_IOC_SETFTYPE:
		file_type = get_fs_long ((long *) arg);

		/* Make sure user is the owner of the file & file is writable */
		if ((current->fsuid != inode->i_uid) && !fsuser())
				return -EPERM;
		if (IS_RDONLY(inode))
			return -EROFS;

		CBM_I(inode)->i_type = file_type;
		inode->i_ctime = CURRENT_TIME;
		inode->i_dirt = 1;		/* inode is dirty */
		return 0;

	/* Get CBM REL file record size */
	case CBM_IOC_GETRECSIZE:
		if (!IS_REL(CBM_I(inode)->i_type))
			return -EINVAL;		/* not REL file */

		if ((err = verify_area (VERIFY_WRITE, (long *) arg, sizeof(long))))
			return err;
		put_fs_long (CBM_I(inode)->i_rec_size, (long *) arg);
		return 0;

	/* Get the file media type (1541 or 1581) */
	case CBM_IOC_GETMEDIA:
		if ((err = verify_area (VERIFY_WRITE, (long *) arg, sizeof(long))))
			return err;
		put_fs_long ((unsigned long) CBM_SB(inode->i_sb)->fs_type, (long *) arg);
		return 0;

	default:
		return -EINVAL;
	}
}
